package com.qucheng.game.data.oldsystem.util;

import org.apache.kafka.connect.data.Field;
import org.apache.kafka.connect.data.Schema;
import org.apache.kafka.connect.data.Struct;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.math.BigDecimal;
import java.nio.ByteBuffer;
import java.nio.charset.StandardCharsets;
import java.time.*;
import java.time.format.DateTimeFormatter;
import java.util.*;

public class CDCStructUtil {
    /**
     * mysql内 date格式直接获取是 int数字，需要用数字加上下面的日期获取真正的时间
     */
    private static final LocalDate BEGIN_DAY = LocalDate.of(1970, 1, 1);
    /**
     * mysql返回的 datetime是 1970-1-1（-28800000）到今天的毫秒数
     */
    private static final Long BEGIN_TIME = LocalDateTime.of(1970, 1, 1, 0, 0).toInstant(ZoneOffset.ofHours(8)).toEpochMilli();

    private static final DateTimeFormatter FORMAT_TIMESTAMP = DateTimeFormatter.ofPattern("yyyy-MM-dd'T'HH:mm:ss'Z'");

    public static <T> T structToBean(Struct struct, Class<T> clazz) throws InstantiationException, IllegalAccessException, NoSuchMethodException, InvocationTargetException {
        Schema schema = struct.schema();
        List<Field> fieldList = schema.fields();
        Map<String, java.lang.reflect.Field> beanFieldMap = parseClass(clazz);

        T t = clazz.newInstance();
        for (Field field : fieldList) {
            java.lang.reflect.Field beanField = beanFieldMap.get(lineToHump(field.name()));
            if (beanField == null) {
                continue;
            }
            Method method = methodOfSet(clazz, beanField);
            method.invoke(t, fieldTransform(struct.get(field), beanField));
        }
        return t;
    }

    public static Map<String, Object> structToMap(Struct struct) {
        Schema schema = struct.schema();
        List<Field> fieldList = schema.fields();
        Map<String, Object> result = new HashMap<>(fieldList.size());

        for (Field field : fieldList) {
            Object obj = struct.get(field);
            if (obj != null) {
                if (field.schema().type() == Schema.Type.INT64 && "io.debezium.time.Timestamp".equals(field.schema().name())) {
                    // datetime类型
                    obj = DateUtil.milliToLocalDateTime(ObjectUtil.objToLong(obj) + BEGIN_TIME);
                } else if (field.schema().type() == Schema.Type.INT32 && "io.debezium.time.Date".equals(field.schema().name())) {
                    // date类型
                    obj = BEGIN_DAY.plusDays(ObjectUtil.objToLong(obj));
                } else if (field.schema().type() == Schema.Type.STRING && "io.debezium.time.ZonedTimestamp".equals(field.schema().name())) {
                    // timestamp类型
                    // System.out.println(field.name() + ":: " + field.schema().name() + ", " + obj.getClass() + ", " + obj);
                    obj = LocalDateTime.parse(ObjectUtil.objToString(obj), FORMAT_TIMESTAMP).plusHours(8);
                } else if (field.schema().type() == Schema.Type.INT64 && "io.debezium.time.MicroTime".equals(field.schema().name())) {
                    // time类型
                    obj = LocalTime.MIN.plusNanos(ObjectUtil.objToLong(obj) * 1000);
                } else if (field.schema().type() == Schema.Type.BYTES && obj instanceof ByteBuffer) {
                    // blob类型
                    obj = ((ByteBuffer) obj).array();
                }
            }
            result.put(field.name(), obj);
        }
        return result;
    }

    private static <T> Map<String, java.lang.reflect.Field> parseClass(Class<T> clazz) {
        List<java.lang.reflect.Field> fieldList = new ArrayList<>();
        Class<?> tempClazz = clazz;
        while (tempClazz != null) {
            java.lang.reflect.Field[] fields = tempClazz.getDeclaredFields();
            if (fields.length != 0) {
                fieldList.addAll(Arrays.asList(fields));
            }
            tempClazz = tempClazz.getSuperclass();
        }
        if (fieldList.size() == 0) {
            throw new RuntimeException("Failed parsed class[" + clazz.getName() + "]. No Field!");
        }
        Map<String, java.lang.reflect.Field> result = new HashMap<>(fieldList.size());
        for (java.lang.reflect.Field field : fieldList) {
            if (Modifier.isFinal(field.getModifiers()) || Modifier.isStatic(field.getModifiers())) {
                continue;
            }
            result.put(field.getName(), field);
        }
        if (result.isEmpty()) {
            throw new RuntimeException("Failed parsed class[" + clazz.getName() + "]. No available Field!");
        }
        return result;
    }

    private static Object fieldTransform(Object obj, java.lang.reflect.Field beanField) {
        if (obj == null) {
            return null;
        }
        Class<?> fieldType = beanField.getType();
        if (obj.getClass() == fieldType) {
            return obj;
        }
        if (fieldType == Integer.class) {
            return ObjectUtil.objToInteger(obj);
        }
        if (fieldType == Long.class) {
            return ObjectUtil.objToLong(obj);
        }
        if (fieldType == String.class) {
            return ObjectUtil.objToString(obj);
        }
        if (fieldType == BigDecimal.class) {
            return ObjectUtil.objToBigDecimal(obj);
        }
        if (fieldType == Boolean.class) {
            return ObjectUtil.objToBoolean(obj);
        }
        if (fieldType == LocalDate.class) {
            if (obj instanceof Integer) {
                return BEGIN_DAY.plusDays((Integer) obj);
            }
            return ObjectUtil.objToLocalDate(obj);
        }
        if (fieldType == LocalDateTime.class) {
            if (obj instanceof Long) {
                // CDC给的时间戳是已经转换过时区的（此处如果用 DateUtil里面的 milliToLocalDateTime()的话，又会 +8小时
                return Instant.ofEpochMilli((Long) obj).atZone(ZoneOffset.ofHours(0)).toLocalDateTime();
            }
            return ObjectUtil.objToLocalDateTime(obj);
        }
        throw new RuntimeException("Failed transform field! Field: " + beanField.getName() + ", Object: " + obj.getClass() + ", ObjectValue: " + obj + ", field: " + fieldType);
    }

    private static <T> Method methodOfSet(Class<T> clazz, java.lang.reflect.Field field) throws NoSuchMethodException {
        String fieldName = field.getName();
        String methodName = "set" + fieldName.substring(0, 1).toUpperCase() + fieldName.substring(1);
        return clazz.getMethod(methodName, field.getType());
    }

    /**
     * 下划线转驼峰
     *
     * @param str
     * @return
     */
    private static String lineToHump(String str) {
        byte[] bytes = str.getBytes(StandardCharsets.UTF_8);
        byte[] result = new byte[str.length()];
        int index = 0;
        for (int i = 0; i < bytes.length; i++) {
            if (bytes[i] != 95) {
                result[index++] = bytes[i];
                continue;
            }
            if (i == bytes.length - 1) {
                result[index++] = bytes[i];
                continue;
            }
            byte nextChar = bytes[++i];
            if (97 <= nextChar && nextChar <= 122) {
                result[index++] = (byte) (nextChar - 32);
            } else {
                result[index++] = nextChar;
            }
        }
        return new String(result, 0, index);
    }
}
